@@ -23,26 +23,34 @@
 #include <QtCore/QDateTime>
 #include <QtCore/QDir>
 #include <QtCore/QTextStream>
+#include <QStandardPaths>
+#include <QJsonDocument>
+#include <QJsonObject>
 
-#include <qjson/parser.h>
-#include <qjson/serializer.h>
-
+#include <kscreen/types.h>
 #include <kscreen/config.h>
 #include <kscreen/output.h>
 #include <kscreen/mode.h>
 #include <kscreen/configmonitor.h>
 #include <kscreen/edid.h>
+#include <kscreen/getconfigoperation.h>
 
-#include <KStandardDirs>
+
+namespace KScreen
+{
+namespace ConfigSerializer
+{
+// Exported private symbol in configserializer_p.h in KScreen
+extern QJsonObject serializeConfig(const KScreen::ConfigPtr &config);
+}
+}
 
 using namespace KScreen;
 
-Console::Console(QObject* parent): QObject(parent)
+Console::Console(const ConfigPtr &config)
+    : QObject()
+    , m_config(config)
 {
-    qDebug() << "START";
-    QDateTime date = QDateTime::currentDateTime();
-    m_config = Config::current();
-    qDebug() << "Config::current() took" << date.msecsTo(QDateTime::currentDateTime()) << "milliseconds";
 }
 
 Console::~Console()
@@ -52,8 +60,6 @@ Console::~Console()
 
 void Console::printConfig()
 {
-    qDebug() << "KScreen version: " << KSCREEN_VERSION;
-
     if (!m_config) {
         qDebug() << "Config is invalid, probably backend couldn't load";
         return;
@@ -63,7 +69,10 @@ void Console::printConfig()
         return;
     }
 
-    connect(m_config, SIGNAL(primaryOutputChanged(KScreen::Output*)), SLOT(primaryOutputChanged(KScreen::Output*)));
+    connect(m_config.data(), &Config::primaryOutputChanged,
+            [&](const OutputPtr &output) {
+                qDebug() << "New primary output: " << output->id() << output->name();
+            });
 
     qDebug() << "Screen:";
     qDebug() << "\tmaxSize:" << m_config->screen()->maxSize();
@@ -71,7 +80,7 @@ void Console::printConfig()
     qDebug() << "\tcurrentSize:" << m_config->screen()->currentSize();
 
     OutputList outputs = m_config->outputs();
-    Q_FOREACH(Output *output, outputs) {
+    Q_FOREACH(const OutputPtr &output, outputs) {
         qDebug() << "\n-----------------------------------------------------\n";
         qDebug() << "Id: " << output->id();
         qDebug() << "Name: " << output->name();
@@ -99,7 +108,7 @@ void Console::printConfig()
         qDebug() << "Modes: ";
 
         ModeList modes = output->modes();
-        Q_FOREACH(Mode* mode, modes) {
+        Q_FOREACH(const ModePtr &mode, modes) {
             qDebug() << "\t" << mode->id() << "  " << mode->name() << " " << mode->size() << " " << mode->refreshRate();
         }
 
@@ -162,32 +171,35 @@ QString Console::typetoString(const Output::Type& type) const
     };
 }
 
+void Console::printJSONConfig()
+{
+    QJsonDocument doc(KScreen::ConfigSerializer::serializeConfig(m_config));
+    qDebug() << doc.toJson(QJsonDocument::Indented);
+}
+
 void Console::printSerializations()
 {
-    QString path = KStandardDirs::locateLocal("data", "kscreen/");
+    QString path = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/kscreen/";
     qDebug() << "Configs in: " << path;
 
     QDir dir(path);
     QStringList files = dir.entryList(QDir::Files);
     qDebug() << "Number of files: " << files.count() << endl;
 
-    bool ok;
-    QJson::Parser parser;
-    QJson::Serializer serializer;
-    serializer.setIndentMode(QJson::IndentFull);
+    QJsonDocument parser;
     Q_FOREACH(const QString fileName, files) {
-        ok = true;
+        QJsonParseError error;
         qDebug() << fileName;
         QFile file(path + "/" + fileName);
         file.open(QFile::ReadOnly);
-        QVariant data = parser.parse(file.readAll(), &ok);
-        if (!ok) {
+        QVariant data = parser.fromJson(file.readAll(), &error);
+        if (error.error != QJsonParseError::NoError) {
             qDebug() << "    " << "can't parse file";
-            qDebug() << "    " << parser.errorString();
+            qDebug() << "    " << error.errorString();
             continue;
         }
 
-        qDebug() << serializer.serialize(data) << endl;
+        qDebug() << parser.toJson(QJsonDocument::Indented) << endl;
     }
 }
 
@@ -201,10 +213,3 @@ void Console::monitorAndPrint()
     monitor();
     connect(ConfigMonitor::instance(), SIGNAL(configurationChanged()), SLOT(printConfig()));
 }
-
-void Console::primaryOutputChanged(Output* output)
-{
-    qDebug() << "New primary output: " << output->id() << output->name();
-}
-
-#include <console.moc>
