@@ -23,32 +23,30 @@
 #include "profilesmodel.h"
 #endif
 
-#include <QtDeclarative/QDeclarativeView>
-#include <QtDeclarative/QDeclarativeEngine>
-
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QSplitter>
-#include <QtGui/QLabel>
-#include <QtCore/QTimer>
-#include <QtCore/QDir>
-#include <QtDBus/QDBusArgument>
+#include <QVBoxLayout>
+#include <QSplitter>
+#include <QLabel>
+#include <QTimer>
+#include <QtCore/qglobal.h>
 
 #include "declarative/qmloutput.h"
 #include "declarative/qmlscreen.h"
-#include "declarative/iconbutton.h"
 #include "utils.h"
 
 #include <kscreen/output.h>
 #include <kscreen/edid.h>
 #include <kscreen/mode.h>
 #include <kscreen/config.h>
+#include <kscreen/getconfigoperation.h>
 
+#include <QtCore/QDir>
+#include <QStandardPaths>
 #include <KLocalizedString>
-#include <KComboBox>
-#include <KPushButton>
-#include <KDebug>
-#include <KStandardDirs>
-#include <KUrl>
+#include <QComboBox>
+#include <QPushButton>
+#include <QQuickView>
+
+#define QML_PATH "kcm_kscreen/qml/"
 
 Widget::Widget(QWidget *parent):
     QWidget(parent),
@@ -56,6 +54,8 @@ Widget::Widget(QWidget *parent):
     mConfig(0),
     mPrevConfig(0)
 {
+    qRegisterMetaType<QQuickView*>();
+
     setMinimumHeight(550);
 
     QVBoxLayout *layout = new QVBoxLayout(this);
@@ -63,10 +63,12 @@ Widget::Widget(QWidget *parent):
     QSplitter *splitter = new QSplitter(Qt::Vertical, this);
     layout->addWidget(splitter);
 
-    m_declarativeView = new QDeclarativeView(this);
-    m_declarativeView->setResizeMode(QDeclarativeView::SizeRootObjectToView);
+    m_declarativeView = new QQuickView();
+    QWidget *container = QWidget::createWindowContainer(m_declarativeView, this);
+    m_declarativeView->setResizeMode(QQuickView::SizeRootObjectToView);
     m_declarativeView->setMinimumHeight(280);
-    splitter->addWidget(m_declarativeView);
+    container->setMinimumHeight(280);
+    splitter->addWidget(container);
 
     QWidget *widget = new QWidget(this);
     splitter->addWidget(widget);
@@ -78,10 +80,11 @@ Widget::Widget(QWidget *parent):
     QHBoxLayout *hbox = new QHBoxLayout;
     vbox->addLayout(hbox);
 
-    mPrimaryCombo = new KComboBox(this);
-    mPrimaryCombo->setSizeAdjustPolicy(QComboBox::QComboBox::AdjustToContents);
+    mPrimaryCombo = new QComboBox(this);
+    mPrimaryCombo->setSizeAdjustPolicy(QComboBox::AdjustToContents);
     mPrimaryCombo->addItem(i18n("No primary screen"));
-    connect(mPrimaryCombo, SIGNAL(currentIndexChanged(int)), SLOT(slotPrimaryChanged(int)));
+    connect(mPrimaryCombo, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+            this, &Widget::slotPrimaryChanged);
 
     hbox->addWidget(new QLabel(i18n("Primary display:")));
     hbox->addWidget(mPrimaryCombo);
@@ -93,7 +96,7 @@ Widget::Widget(QWidget *parent):
 
     connect(mProfilesModel, SIGNAL(modelUpdated()),
             this, SLOT(slotProfilesUpdated()));
-    mProfilesCombo = new KComboBox(this);
+    mProfilesCombo = new QComboBox(this);
     mProfilesCombo->setModel(mProfilesModel);
     mProfilesCombo->setSizeAdjustPolicy(QComboBox::AdjustToContents);
     hbox->addWidget(new QLabel(i18n("Active profile")));
@@ -101,15 +104,22 @@ Widget::Widget(QWidget *parent):
 #endif
 
     m_controlPanel = new ControlPanel(this);
-    connect(m_controlPanel, SIGNAL(changed()), this, SIGNAL(changed()));
+    connect(m_controlPanel, &ControlPanel::changed,
+            this, &Widget::changed);
     vbox->addWidget(m_controlPanel);
 
-    mUnifyButton = new KPushButton(i18n("Unify outputs"), this);
-    connect(mUnifyButton, SIGNAL(clicked(bool)), this, SLOT(slotUnifyOutputs()));
+    mUnifyButton = new QPushButton(i18n("Unify outputs"), this);
+    connect(mUnifyButton, &QPushButton::clicked,
+            [&](bool clicked) {
+                Q_UNUSED(clicked);
+                slotUnifyOutputs();
+            });
+
     vbox->addWidget(mUnifyButton);
 
     mOutputTimer = new QTimer(this);
-    connect(mOutputTimer, SIGNAL(timeout()), SLOT(clearOutputIdentifiers()));
+    connect(mOutputTimer, &QTimer::timeout,
+            this, &Widget::clearOutputIdentifiers);
 
     loadQml();
 }
@@ -119,33 +129,56 @@ Widget::~Widget()
     clearOutputIdentifiers();
 }
 
-void Widget::setConfig(KScreen::Config *config)
+bool Widget::eventFilter(QObject* object, QEvent* event)
+{
+    if (event->type() == QEvent::Resize) {
+        if (mOutputIdentifiers.contains(qobject_cast<QQuickView*>(object))) {
+            QResizeEvent *e = static_cast<QResizeEvent*>(event);
+            const QRect screenSize = object->property("screenSize").toRect();
+            QRect geometry(QPoint(0, 0), e->size());
+            geometry.moveCenter(screenSize.center());
+            static_cast<QQuickView*>(object)->setGeometry(geometry);
+            // Pass the event further
+        }
+    }
+
+    return QObject::eventFilter(object, event);
+}
+
+
+void Widget::setConfig(const KScreen::ConfigPtr &config)
 {
     if (mConfig) {
-        Q_FOREACH (KScreen::Output *output, mConfig->outputs()) {
-            disconnect(output, SIGNAL(isConnectedChanged()), this, SLOT(slotOutputConnectedChanged()));
-            disconnect(output, SIGNAL(isEnabledChanged()), this, SLOT(slotOutputEnabledChanged()));
-            disconnect(output, SIGNAL(isPrimaryChanged()), this, SLOT(slotOutputPrimaryChanged()));
-            disconnect(output, SIGNAL(posChanged()), this, SIGNAL(changed()));
+        Q_FOREACH (const KScreen::OutputPtr &output, mConfig->outputs()) {
+            disconnect(output.data(), &KScreen::Output::isConnectedChanged,
+                       this, &Widget::slotOutputConnectedChanged);
+            disconnect(output.data(), &KScreen::Output::isEnabledChanged,
+                       this, &Widget::slotOutputEnabledChanged);
+            disconnect(output.data(), &KScreen::Output::isPrimaryChanged,
+                       this, &Widget::slotOutputPrimaryChanged);
+            disconnect(output.data(), &KScreen::Output::posChanged,
+                       this, &Widget::changed);
         }
-
-        delete mConfig;
     }
 
     mConfig = config;
     mScreen->setConfig(mConfig);
     m_controlPanel->setConfig(mConfig);
-    Q_FOREACH (KScreen::Output *output, mConfig->outputs()) {
-        connect(output, SIGNAL(isConnectedChanged()), this, SLOT(slotOutputConnectedChanged()));
-        connect(output, SIGNAL(isEnabledChanged()), this, SLOT(slotOutputEnabledChanged()));
-        connect(output, SIGNAL(isPrimaryChanged()), this, SLOT(slotOutputPrimaryChanged()));
-        connect(output, SIGNAL(posChanged()), this, SIGNAL(changed()));
+    Q_FOREACH (const KScreen::OutputPtr &output, mConfig->outputs()) {
+        connect(output.data(), &KScreen::Output::isConnectedChanged,
+                this, &Widget::slotOutputConnectedChanged);
+        connect(output.data(), &KScreen::Output::isEnabledChanged,
+                this, &Widget::slotOutputEnabledChanged);
+        connect(output.data(), &KScreen::Output::isPrimaryChanged,
+                this, &Widget::slotOutputPrimaryChanged);
+        connect(output.data(), &KScreen::Output::posChanged,
+                this, &Widget::changed);
     }
 
     initPrimaryCombo();
 }
 
-KScreen::Config *Widget::currentConfig() const
+KScreen::ConfigPtr Widget::currentConfig() const
 {
     return mConfig;
 }
@@ -154,36 +187,26 @@ void Widget::loadQml()
 {
     qmlRegisterType<QMLOutput>("org.kde.kscreen", 1, 0, "QMLOutput");
     qmlRegisterType<QMLScreen>("org.kde.kscreen", 1, 0, "QMLScreen");
-    qmlRegisterType<IconButton>("org.kde.kscreen", 1, 0, "IconButton");
 
     qmlRegisterType<KScreen::Output>("org.kde.kscreen", 1, 0, "KScreenOutput");
     qmlRegisterType<KScreen::Edid>("org.kde.kscreen", 1, 0, "KScreenEdid");
     qmlRegisterType<KScreen::Mode>("org.kde.kscreen", 1, 0, "KScreenMode");
 
     //const QString file = QDir::currentPath() + "/main.qml";
-    const QString file = KStandardDirs::locate("data", QLatin1String("kcm_kscreen/qml/main.qml"));
-    QStringListIterator paths(KGlobal::dirs()->findDirs("module", "imports"));
-    paths.toBack();
-    while (paths.hasPrevious()) {
-        m_declarativeView->engine()->addImportPath(paths.previous());
-    }
+    const QString file = QStandardPaths::locate(QStandardPaths::QStandardPaths::GenericDataLocation, QStringLiteral("kcm_kscreen/qml/main.qml"));
     m_declarativeView->setSource(QUrl::fromLocalFile(file));
 
-    QGraphicsObject *rootObject = m_declarativeView->rootObject();
+    QQuickItem* rootObject = m_declarativeView->rootObject();
     mScreen = rootObject->findChild<QMLScreen*>(QLatin1String("outputView"));
     if (!mScreen) {
         return;
     }
+    mScreen->setEngine(m_declarativeView->engine());
 
-    connect(mScreen, SIGNAL(focusedOutputChanged(QMLOutput*)),
-            this, SLOT(slotFocusedOutputChanged(QMLOutput*)));
+    connect(mScreen, &QMLScreen::focusedOutputChanged,
+            this, &Widget::slotFocusedOutputChanged);
     connect(rootObject->findChild<QObject*>("identifyButton"), SIGNAL(clicked()),
-            this, SLOT(slotIdentifyOutputs()));
-
-
-#ifndef WITH_PROFILES
-    setConfig(KScreen::Config::current());
-#endif
+            this, SLOT(slotIdentifyButtonClicked()));
 }
 
 void Widget::initPrimaryCombo()
@@ -192,7 +215,7 @@ void Widget::initPrimaryCombo()
     mPrimaryCombo->clear();
     mPrimaryCombo->addItem(i18n("No primary output"));
 
-    Q_FOREACH (KScreen::Output *output, mConfig->outputs()) {
+    Q_FOREACH (const KScreen::OutputPtr &output, mConfig->outputs()) {
         if (!output->isConnected() || !output->isEnabled()) {
             continue;
         }
@@ -208,7 +231,7 @@ void Widget::initPrimaryCombo()
 
 void Widget::slotFocusedOutputChanged(QMLOutput *output)
 {
-    m_controlPanel->activateOutput(output->output());
+    m_controlPanel->activateOutput(output->outputPtr());
 }
 
 void Widget::slotOutputPrimaryChanged()
@@ -227,7 +250,7 @@ void Widget::slotOutputPrimaryChanged()
 void Widget::slotPrimaryChanged(int index)
 {
     const int id = mPrimaryCombo->itemData(index).toInt();
-    Q_FOREACH (KScreen::Output *output, mConfig->outputs()) {
+    for (KScreen::OutputPtr &output : mConfig->outputs()) {
         output->blockSignals(true);
         output->setPrimary(output->id() == id);
         output->blockSignals(false);
@@ -253,7 +276,7 @@ void Widget::slotOutputConnectedChanged()
 void Widget::slotOutputEnabledChanged()
 {
     int enabledOutputsCnt = 0;
-    Q_FOREACH (KScreen::Output *output, mConfig->outputs()) {
+    Q_FOREACH (const KScreen::OutputPtr &output, mConfig->outputs()) {
         if (output->isEnabled()) {
             ++enabledOutputsCnt;
         }
@@ -295,17 +318,13 @@ void Widget::slotUnifyOutputs()
 
     if (base->isCloneMode()) {
         setConfig(mPrevConfig);
-        mPrevConfig = 0;
+        mPrevConfig.clear();
 
         mPrimaryCombo->setEnabled(true);
         mUnifyButton->setText(i18n("Unify Outputs"));
     } else {
         // Clone the current config, so that we can restore it in case user
         // breaks the cloning
-        if (mPrevConfig) {
-            delete mPrevConfig;
-        }
-
         mPrevConfig = mConfig->clone();
 
         Q_FOREACH (QMLOutput *output, mScreen->outputs()) {
@@ -314,7 +333,7 @@ void Widget::slotUnifyOutputs()
             }
 
             if (!output->output()->isEnabled()) {
-                output->hide();
+                output->setVisible(false);
                 continue;
             }
 
@@ -330,7 +349,7 @@ void Widget::slotUnifyOutputs()
             if (base != output) {
                 clones << output->output()->id();
                 output->setCloneOf(base);
-                output->hide();
+                output->setVisible(false);
             }
         }
 
@@ -340,7 +359,7 @@ void Widget::slotUnifyOutputs()
         mScreen->updateOutputsPlacement();
 
         mPrimaryCombo->setEnabled(false);
-        m_controlPanel->setUnifiedOutput(base->output());
+        m_controlPanel->setUnifiedOutput(base->outputPtr());
 
         mUnifyButton->setText(i18n("Break unified outputs"));
     }
@@ -377,14 +396,16 @@ void Widget::slotProfileChanged(int index)
     config->setOutputs(outputList);
 
     setConfig(config);
+#else
+    Q_UNUSED(index)
 #endif
 }
 
 // FIXME: Copy-pasted from KDED's Serializer::findOutput()
-KScreen::Output *Widget::findOutput(KScreen::Config *config, const QVariantMap &info)
+KScreen::OutputPtr Widget::findOutput(const KScreen::ConfigPtr &config, const QVariantMap &info)
 {
     KScreen::OutputList outputs = config->outputs();
-    Q_FOREACH(KScreen::Output * output, outputs) {
+    Q_FOREACH(const KScreen::OutputPtr &output, outputs) {
         if (!output->isConnected()) {
             continue;
         }
@@ -405,8 +426,8 @@ KScreen::Output *Widget::findOutput(KScreen::Config *config, const QVariantMap &
         QVariantMap modeSize = modeInfo["size"].toMap();
         QSize size(modeSize["width"].toInt(), modeSize["height"].toInt());
 
-        KScreen::ModeList modes = output->modes();
-        Q_FOREACH(KScreen::Mode * mode, modes) {
+        const KScreen::ModeList modes = output->modes();
+        Q_FOREACH(const KScreen::ModePtr &mode, modes) {
             if (mode->size() != size) {
                 continue;
             }
@@ -420,7 +441,7 @@ KScreen::Output *Widget::findOutput(KScreen::Config *config, const QVariantMap &
         return output;
     }
 
-    return 0;
+    return KScreen::OutputPtr();
 }
 
 void Widget::slotProfilesAboutToUpdate()
@@ -451,38 +472,43 @@ void Widget::clearOutputIdentifiers()
     mOutputIdentifiers.clear();
 }
 
-void Widget::slotIdentifyOutputs()
+void Widget::slotIdentifyButtonClicked()
 {
-    kDebug();
-    kDebug() << "!";
-    const QString file = KStandardDirs::locate("data", QLatin1String("kcm_kscreen/qml/OutputIdentifier.qml"));
-    QStringListIterator paths(KGlobal::dirs()->findDirs("module", "imports"));
-    paths.toBack();
+    connect(new KScreen::GetConfigOperation(), &KScreen::GetConfigOperation::finished,
+            this, &Widget::slotIdentifyOutputs);
+}
+
+void Widget::slotIdentifyOutputs(KScreen::ConfigOperation *op)
+{
+    if (op->hasError()) {
+        return;
+    }
+
+    const KScreen::ConfigPtr config = qobject_cast<KScreen::GetConfigOperation*>(op)->config();
+
+    const QString qmlPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String(QML_PATH "OutputIdentifier.qml"));
 
     mOutputTimer->stop();
     clearOutputIdentifiers();
 
     /* Obtain the current active configuration from KScreen */
-    KScreen::OutputList outputs = KScreen::Config::current()->outputs();
-    Q_FOREACH (KScreen::Output *output, outputs) {
+    Q_FOREACH (const KScreen::OutputPtr &output, config->outputs()) {
         if (!output->isConnected() || !output->currentMode()) {
             continue;
         }
 
-        KScreen::Mode *mode = output->currentMode();
+        const KScreen::ModePtr mode = output->currentMode();
 
-        QDeclarativeView *view = new QDeclarativeView();
-        view->setWindowFlags(Qt::X11BypassWindowManagerHint | Qt::FramelessWindowHint);
-        view->setResizeMode(QDeclarativeView::SizeViewToRootObject);
-        while (paths.hasPrevious()) {
-            view->engine()->addImportPath(paths.previous());
-        }
-        view->setSource(QUrl::fromLocalFile(file));
+        QQuickView *view = new QQuickView();
 
+        view->setFlags(Qt::X11BypassWindowManagerHint | Qt::FramelessWindowHint);
+        view->setResizeMode(QQuickView::SizeViewToRootObject);
+        view->setSource(QUrl::fromLocalFile(qmlPath));
+        view->installEventFilter(this);
 
-        QDeclarativeItem *rootObj = dynamic_cast<QDeclarativeItem*>(view->rootObject());
+        QQuickItem *rootObj = view->rootObject();
         if (!rootObj) {
-            kWarning() << "Failed to obtain root item";
+            qWarning() << "Failed to obtain root item";
             continue;
         }
         QSize realSize;
@@ -491,22 +517,15 @@ void Widget::slotIdentifyOutputs()
         } else {
             realSize = QSize(mode->size().height(), mode->size().width());
         }
-        rootObj->setProperty("displayName", Utils::outputName(output));
+        rootObj->setProperty("outputName", Utils::outputName(output));
         rootObj->setProperty("modeName", Utils::sizeToString(realSize));
-
-        const QRect outputRect(output->pos(), realSize);
-        QRect geometry(QPoint(0, 0), view->sizeHint());
-        geometry.moveCenter(outputRect.center());
-        view->setGeometry(geometry);
-
+        view->setProperty("screenSize", QRect(output->pos(), realSize));
         mOutputIdentifiers << view;
     }
 
-    Q_FOREACH (QWidget *widget, mOutputIdentifiers) {
-        widget->show();
+    Q_FOREACH (QQuickView *view, mOutputIdentifiers) {
+        view->show();
     }
 
     mOutputTimer->start(2500);
 }
-
-#include "widget.moc"
